@@ -10,6 +10,7 @@ import (
 	"github.com/percona/percona-backup-mongodb/pbm/backup"
 	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/restore"
+	"github.com/percona/percona-backup-mongodb/pbm/storage"
 )
 
 type currentBackup struct {
@@ -46,7 +47,13 @@ func (a *Agent) CancelBackup() {
 }
 
 // Backup starts backup
-func (a *Agent) Backup(cmd pbm.BackupCmd, opid pbm.OPID, ep pbm.Epoch) {
+func (a *Agent) Backup(cmd *pbm.BackupCmd, opid pbm.OPID, ep pbm.Epoch) {
+	if cmd == nil {
+		l := a.log.NewEvent(string(pbm.CmdBackup), "", opid.String(), ep.TS())
+		l.Error("missed command")
+		return
+	}
+
 	l := a.log.NewEvent(string(pbm.CmdBackup), cmd.Name, opid.String(), ep.TS())
 
 	nodeInfo, err := a.node.GetInfo()
@@ -167,7 +174,7 @@ func (a *Agent) Backup(cmd pbm.BackupCmd, opid pbm.OPID, ep pbm.Epoch) {
 
 	ctx, cancel := context.WithCancel(context.Background())
 	a.setBcp(&currentBackup{
-		header: &cmd,
+		header: cmd,
 		cancel: cancel,
 	})
 	l.Info("backup started")
@@ -257,12 +264,21 @@ func (a *Agent) waitNomination(bcp, rs, node string, l *log.Event) (got bool, er
 	}
 }
 
-func (a *Agent) Restore(r pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch) {
-	l := a.log.NewEvent(string(pbm.CmdRestore), r.BackupName, opid.String(), ep.TS())
+func (a *Agent) Restore(r *pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch) {
+	if r == nil {
+		l := a.log.NewEvent(string(pbm.CmdRestore), "", opid.String(), ep.TS())
+		l.Error("missed command")
+		return
+	}
 
+	l := a.log.NewEvent(string(pbm.CmdRestore), r.Name, opid.String(), ep.TS())
+
+	l.Info("backup: %s", r.BackupName)
+
+	var stg storage.Storage
 	bcp, err := a.pbm.GetBackupMeta(r.BackupName)
 	if errors.Is(err, pbm.ErrNotFound) {
-		stg, err := a.pbm.GetStorage(l)
+		stg, err = a.pbm.GetStorage(l)
 		if err != nil {
 			l.Error("get storage: %v", err)
 			return
@@ -291,7 +307,7 @@ func (a *Agent) Restore(r pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch) {
 }
 
 // restoreLogical starts the restore
-func (a *Agent) restoreLogical(r pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch, l *log.Event) error {
+func (a *Agent) restoreLogical(r *pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch, l *log.Event) error {
 	nodeInfo, err := a.node.GetInfo()
 	if err != nil {
 		return errors.Wrap(err, "get node info")
@@ -351,7 +367,7 @@ func (a *Agent) restoreLogical(r pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch, l
 }
 
 // restoreLogical starts the restore
-func (a *Agent) restorePhysical(r pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch, l *log.Event) error {
+func (a *Agent) restorePhysical(r *pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch, l *log.Event) error {
 	nodeInfo, err := a.node.GetInfo()
 	if err != nil {
 		return errors.Wrap(err, "get node info")
@@ -362,6 +378,9 @@ func (a *Agent) restorePhysical(r pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch, l
 		return errors.Wrap(err, "init physical backup")
 	}
 
+	// physical restore runs on all nodes in the replset
+	// so we try lock only on primary only to be sure there
+	// is no concurrent operation running.
 	var lock *pbm.Lock
 	if nodeInfo.IsPrimary {
 		epts := ep.TS()
@@ -383,8 +402,6 @@ func (a *Agent) restorePhysical(r pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch, l
 		}
 	}
 
-	// not to logs flood with errors when mongo went down
-	a.closeCMD <- struct{}{}
 	if lock != nil {
 		// Don't care about errors. Anyway, the lock gonna disappear after the
 		// restore. And the commands stream is down as well.
@@ -394,7 +411,7 @@ func (a *Agent) restorePhysical(r pbm.RestoreCmd, opid pbm.OPID, ep pbm.Epoch, l
 	}
 
 	l.Info("restore started")
-	err = rstr.Snapshot(r, opid, l)
+	err = rstr.Snapshot(r, opid, l, a.closeCMD)
 	l.Info("restore finished %v", err)
 	if err != nil {
 		if errors.Is(err, restore.ErrNoDataForShard) {
