@@ -480,27 +480,33 @@ func (s storageStat) String() string {
 
 	sort.Slice(s.Snapshot, func(i, j int) bool {
 		a, b := s.Snapshot[i], s.Snapshot[j]
-		return a.StateTS > b.StateTS
+		return a.RestoreTS > b.RestoreTS
 	})
 
 	for _, sn := range s.Snapshot {
 		var status string
 		switch sn.Status {
 		case pbm.StatusDone:
-			status = fmt.Sprintf("[complete: %s]", fmtTS(sn.StateTS))
+			status = fmt.Sprintf("[restore_to_time: %s]", fmtTS(sn.RestoreTS))
 		case pbm.StatusCancelled:
-			status = fmt.Sprintf("[!canceled: %s]", fmtTS(sn.StateTS))
+			status = fmt.Sprintf("[!canceled: %s]", fmtTS(sn.RestoreTS))
 		case pbm.StatusError:
 			if errors.Is(sn.Err, errIncompatible) {
-				status = fmt.Sprintf("[incompatible: %s] [%s]", sn.Err.Error(), fmtTS(sn.StateTS))
+				status = fmt.Sprintf("[incompatible: %s] [%s]", sn.Err.Error(), fmtTS(sn.RestoreTS))
 			} else {
-				status = fmt.Sprintf("[ERROR: %s] [%s]", sn.Err.Error(), fmtTS(sn.StateTS))
+				status = fmt.Sprintf("[ERROR: %s] [%s]", sn.Err.Error(), fmtTS(sn.RestoreTS))
 			}
 		default:
-			status = fmt.Sprintf("[running: %s / %s]", sn.Status, fmtTS(sn.StateTS))
+			status = fmt.Sprintf("[running: %s / %s]", sn.Status, fmtTS(sn.RestoreTS))
 		}
 
-		ret += fmt.Sprintf("    %s %s <%s> %s\n", sn.Name, fmtSize(sn.Size), sn.Type, status)
+		kind := string(sn.Type)
+		if len(sn.Namespaces) != 0 {
+			kind += ", selective"
+		}
+
+		ret += fmt.Sprintf("    %s %s <%s> %s\n",
+			sn.Name, fmtSize(sn.Size), kind, status)
 	}
 
 	if len(s.PITR.Ranges) == 0 {
@@ -539,7 +545,7 @@ func getStorageStat(cn *pbm.PBM, rsMap map[string]string) (fmt.Stringer, error)
 
 	s.Type = cfg.Storage.Typ()
 
-	if cfg.Storage.Type == pbm.StorageS3 {
+	if cfg.Storage.Type == storage.S3 {
 		s.Region = cfg.Storage.S3.Region
 	}
 	s.Path = cfg.Storage.Path()
@@ -576,8 +582,9 @@ func getStorageStat(cn *pbm.PBM, rsMap map[string]string) (fmt.Stringer, error)
 	for _, bcp := range bcps {
 		snpsht := snapshotStat{
 			Name:       bcp.Name,
+			Namespaces: bcp.Namespaces,
 			Status:     bcp.Status,
-			StateTS:    bcp.LastTransitionTS,
+			RestoreTS:  bcp.LastTransitionTS,
 			PBMVersion: bcp.PBMVersion,
 			Type:       bcp.Type,
 			Err:        bcp.Error(),
@@ -590,18 +597,7 @@ func getStorageStat(cn *pbm.PBM, rsMap map[string]string) (fmt.Stringer, error)
 			}
 			fallthrough
 		case pbm.StatusDone:
-			snpsht.StateTS = int64(bcp.LastWriteTS.T)
-			var err error
-			switch bcp.Type {
-			case pbm.PhysicalBackup:
-				snpsht.Size, err = getPhysSnapshotSize(&bcp, stg)
-			default:
-				snpsht.Size, err = getSnapshotSize(bcp.Replsets, stg)
-			}
-			if err != nil {
-				snpsht.Err = err
-				snpsht.Status = pbm.StatusError
-			}
+			snpsht.RestoreTS = int64(bcp.LastWriteTS.T)
 		case pbm.StatusCancelled:
 			// leave as it is, not to rewrite status with the `stuck` error
 		default:
@@ -610,6 +606,13 @@ func getStorageStat(cn *pbm.PBM, rsMap map[string]string) (fmt.Stringer, error)
 				snpsht.Status = pbm.StatusError
 			}
 		}
+
+		snpsht.Size, err = getBackupSize(&bcp, stg)
+		if err != nil {
+			snpsht.Err = err
+			snpsht.Status = pbm.StatusError
+		}
+
 		s.Snapshot = append(s.Snapshot, snpsht)
 	}
 
@@ -688,7 +691,35 @@ func getPITRranges(cn *pbm.PBM, stg storage.Storage, bcps []pbm.BackupMeta, rsMa
 	return &pitrRanges{Ranges: pr, Size: size}, nil
 }
 
-func getSnapshotSize(rsets []pbm.BackupReplset, stg storage.Storage) (s int64, err error) {
+func getBackupSize(bcp *pbm.BackupMeta, stg storage.Storage) (s int64, err error) {
+	if bcp.Size > 0 {
+		return bcp.Size, nil
+	}
+	return getLegacySnapshotSize(bcp.Replsets, bcp.Type, stg)
+}
+
+func getLegacySnapshotSize(rsets []pbm.BackupReplset, typ pbm.BackupType, stg storage.Storage) (s int64, err error) {
+	switch typ {
+	case pbm.LogicalBackup:
+		return getLegacyLogicalSize(rsets, stg)
+	case pbm.PhysicalBackup:
+		return getLegacyPhysSize(rsets, stg)
+	default:
+		return 0, errors.Errorf("unknown backup type %s", typ)
+	}
+}
+
+func getLegacyPhysSize(rsets []pbm.BackupReplset, stg storage.Storage) (s int64, err error) {
+	for _, rs := range rsets {
+		for _, f := range rs.Files {
+			s += f.StgSize
+		}
+	}
+
+	return s, nil
+}
+
+func getLegacyLogicalSize(rsets []pbm.BackupReplset, stg storage.Storage) (s int64, err error) {
 	for _, rs := range rsets {
 		ds, err := stg.FileStat(rs.DumpName)
 		if err != nil {
@@ -704,13 +735,3 @@ func getSnapshotSize(rsets []pbm.BackupReplset, stg storage.Storage) (s int64, e
 
 	return s, nil
 }
-
-func getPhysSnapshotSize(bcp *pbm.BackupMeta, stg storage.Storage) (s int64, err error) {
-	for _, rs := range bcp.Replsets {
-		for _, f := range rs.Files {
-			s += f.StgSize
-		}
-	}
-
-	return s, nil
-}
