@@ -18,6 +18,7 @@ import (
 	"go.mongodb.org/mongo-driver/x/bsonx/bsoncore"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
+	"github.com/percona/percona-backup-mongodb/pbm/compress"
 	plog "github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 )
@@ -155,7 +156,7 @@ func (bc *BackupCursor) Close() {
 	}
 }
 
-func (b *Backup) doPhysical(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, rsMeta *pbm.BackupReplset, inf *pbm.NodeInfo, stg storage.Storage, l *plog.Event) error {
+func (b *Backup) doPhysical(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPID, rsMeta *pbm.BackupReplset, inf *pbm.NodeInfo, stg storage.Storage, l *plog.Event) error {
 	cursor := NewBackupCursor(b.node, l)
 	defer cursor.Close()
 
@@ -171,6 +172,17 @@ func (b *Backup) doPhysical(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPI
 		return errors.Wrap(err, "get shard's last write ts")
 	}
 
+	defOpts := new(pbm.MongodOpts)
+	defOpts.Storage.WiredTiger.EngineConfig.JournalCompressor = "snappy"
+	defOpts.Storage.WiredTiger.CollectionConfig.BlockCompressor = "snappy"
+	defOpts.Storage.WiredTiger.IndexConfig.PrefixCompression = true
+
+	mopts, err := b.node.GetOpts(defOpts)
+	if err != nil {
+		return errors.Wrap(err, "get mongod options")
+	}
+
+	rsMeta.MongodOpts = mopts
 	rsMeta.Status = pbm.StatusRunning
 	rsMeta.FirstWriteTS = bcur.Meta.OplogEnd.TS
 	rsMeta.LastWriteTS = lwts
@@ -240,6 +252,16 @@ func (b *Backup) doPhysical(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPI
 		return errors.Wrap(err, "set shard's files list")
 	}
 
+	size := int64(0)
+	for _, f := range rsMeta.Files {
+		size += f.StgSize
+	}
+
+	err = b.cn.IncBackupSize(ctx, bcp.Name, size)
+	if err != nil {
+		return errors.Wrap(err, "inc backup size")
+	}
+
 	return nil
 }
 
@@ -272,7 +294,7 @@ func (id *UUID) IsZero() bool {
 	return bytes.Equal(id.UUID[:], uuid.Nil[:])
 }
 
-func writeFile(ctx context.Context, src, dst string, stg storage.Storage, compression pbm.CompressionType, compressLevel *int, l *plog.Event) (*pbm.File, error) {
+func writeFile(ctx context.Context, src, dst string, stg storage.Storage, compression compress.CompressionType, compressLevel *int, l *plog.Event) (*pbm.File, error) {
 	fstat, err := os.Stat(src)
 	if err != nil {
 		return nil, errors.Wrap(err, "get file stat")
@@ -282,7 +304,7 @@ func writeFile(ctx context.Context, src, dst string, stg storage.Storage, compre
 
 	dst += compression.Suffix()
 
-	_, err = Upload(ctx, &file{src}, stg, compression, compressLevel, dst, int(fstat.Size()))
+	_, err = Upload(ctx, &file{src}, stg, compression, compressLevel, dst, fstat.Size())
 	if err != nil {
 		return nil, errors.Wrap(err, "upload file")
 	}
