@@ -44,11 +44,12 @@ type NodeInfo struct {
 	ClusterTime                  *ClusterTime         `bson:"$clusterTime,omitempty"`
 	ConfigServerState            *ConfigServerState   `bson:"$configServerState,omitempty"`
 	OperationTime                *primitive.Timestamp `bson:"operationTime,omitempty"`
+	opts                         MongodOpts
 }
 
 // IsSharded returns true is replset is part sharded cluster
 func (i *NodeInfo) IsSharded() bool {
-	return i.SetName != "" && (i.ConfigServerState != nil || i.ConfigSvr == 2)
+	return i.SetName != "" && (i.ConfigServerState != nil || i.opts.Sharding.ClusterRole != "" || i.ConfigSvr == 2)
 }
 
 // IsLeader returns true if node can act as backup leader (it's configsrv or non shareded rs)
@@ -230,12 +231,54 @@ func (b *BalancerStatus) IsOn() bool {
 
 type MongodOpts struct {
 	Net struct {
-		BindIp string `bson:"bindIp" json:"bindIp"`
-		Port   int    `bson:"port" json:"port"`
+		BindIp string `bson:"bindIp" json:"bindIp" yaml:"bindIp"`
+		Port   int    `bson:"port" json:"port" yaml:"port"`
 	} `bson:"net" json:"net"`
-	Storage struct {
-		DBpath string `bson:"dbPath" json:"dbPath"`
-	} `bson:"storage" json:"storage"`
+	Sharding struct {
+		ClusterRole string `bson:"clusterRole" json:"clusterRole" yaml:"-"`
+	} `bson:"sharding" json:"sharding" yaml:"-"`
+	Storage  MongodOptsStorage `bson:"storage" json:"storage" yaml:"storage"`
+	Security *MongodOptsSec    `bson:"security,omitempty" json:"security,omitempty" yaml:"security,omitempty"`
+}
+
+type MongodOptsSec struct {
+	EnableEncryption     *bool   `bson:"enableEncryption,omitempty" json:"enableEncryption,omitempty" yaml:"enableEncryption,omitempty"`
+	EncryptionCipherMode *string `bson:"encryptionCipherMode,omitempty" json:"encryptionCipherMode,omitempty" yaml:"encryptionCipherMode,omitempty"`
+	EncryptionKeyFile    *string `bson:"encryptionKeyFile,omitempty" json:"encryptionKeyFile,omitempty" yaml:"encryptionKeyFile,omitempty"`
+	RelaxPermChecks      *bool   `bson:"relaxPermChecks,omitempty" json:"relaxPermChecks,omitempty" yaml:"relaxPermChecks,omitempty"`
+	Vault                *struct {
+		ServerName   *string `bson:"serverName,omitempty" json:"serverName,omitempty" yaml:"serverName,omitempty"`
+		Port         *int    `bson:"port,omitempty" json:"port,omitempty" yaml:"port,omitempty"`
+		TokenFile    *string `bson:"tokenFile,omitempty" json:"tokenFile,omitempty" yaml:"tokenFile,omitempty"`
+		Secret       *string `bson:"secret,omitempty" json:"secret,omitempty" yaml:"secret,omitempty"`
+		ServerCAFile *string `bson:"serverCAFile,omitempty" json:"serverCAFile,omitempty" yaml:"serverCAFile,omitempty"`
+	} `bson:"vault,omitempty" json:"vault,omitempty" yaml:"vault,omitempty"`
+	KMIP *struct {
+		ServerName                *string `bson:"serverName,omitempty" json:"serverName,omitempty" yaml:"serverName,omitempty"`
+		Port                      *int    `bson:"port,omitempty" json:"port,omitempty" yaml:"port,omitempty"`
+		ClientCertificateFile     *string `bson:"clientCertificateFile,omitempty" json:"clientCertificateFile,omitempty" yaml:"clientCertificateFile,omitempty"`
+		ClientKeyFile             *string `bson:"clientKeyFile,omitempty" json:"clientKeyFile,omitempty" yaml:"clientKeyFile,omitempty"`
+		ServerCAFile              *string `bson:"serverCAFile,omitempty" json:"serverCAFile,omitempty" yaml:"serverCAFile,omitempty"`
+		KeyIdentifier             *string `bson:"keyIdentifier,omitempty" json:"keyIdentifier,omitempty" yaml:"keyIdentifier,omitempty"`
+		ClientCertificatePassword *string `bson:"clientCertificatePassword,omitempty" json:"-" yaml:"clientCertificatePassword,omitempty"`
+	} `bson:"kmip,omitempty" json:"kmip,omitempty" yaml:"kmip,omitempty"`
+}
+
+type MongodOptsStorage struct {
+	DirectoryPerDB bool   `bson:"directoryPerDB" json:"directoryPerDB" yaml:"directoryPerDB"`
+	DBpath         string `bson:"dbPath" json:"dbPath" yaml:"dbPath"`
+	WiredTiger     struct {
+		EngineConfig struct {
+			JournalCompressor   string `bson:"journalCompressor" json:"journalCompressor" yaml:"journalCompressor"`
+			DirectoryForIndexes bool   `bson:"directoryForIndexes" json:"directoryForIndexes" yaml:"directoryForIndexes"`
+		} `bson:"engineConfig" json:"engineConfig" yaml:"engineConfig"`
+		CollectionConfig struct {
+			BlockCompressor string `bson:"blockCompressor" json:"blockCompressor" yaml:"blockCompressor"`
+		} `bson:"collectionConfig" json:"collectionConfig" yaml:"collectionConfig"`
+		IndexConfig struct {
+			PrefixCompression bool `bson:"prefixCompression" json:"prefixCompression" yaml:"prefixCompression"`
+		} `bson:"indexConfig" json:"indexConfig" yaml:"indexConfig"`
+	} `bson:"wiredTiger" json:"wiredTiger" yaml:"wiredTiger"`
 }
 
 type RSConfig struct {
