@@ -19,6 +19,7 @@ import (
 	"go.mongodb.org/mongo-driver/mongo/readpref"
 	"go.mongodb.org/mongo-driver/mongo/writeconcern"
 
+	"github.com/percona/percona-backup-mongodb/pbm/compress"
 	"github.com/percona/percona-backup-mongodb/pbm/log"
 )
 
@@ -104,15 +105,15 @@ func (c Command) String() string {
 type OPID primitive.ObjectID
 
 type Cmd struct {
-	Cmd        Command         `bson:"cmd"`
-	Backup     BackupCmd       `bson:"backup,omitempty"`
-	Restore    RestoreCmd      `bson:"restore,omitempty"`
-	Replay     ReplayCmd       `bson:"replay,omitempty"`
-	PITRestore PITRestoreCmd   `bson:"pitrestore,omitempty"`
-	Delete     DeleteBackupCmd `bson:"delete,omitempty"`
-	DeletePITR DeletePITRCmd   `bson:"deletePitr,omitempty"`
-	TS         int64           `bson:"ts"`
-	OPID       OPID            `bson:"-"`
+	Cmd        Command          `bson:"cmd"`
+	Backup     *BackupCmd       `bson:"backup,omitempty"`
+	Restore    *RestoreCmd      `bson:"restore,omitempty"`
+	Replay     *ReplayCmd       `bson:"replay,omitempty"`
+	PITRestore *PITRestoreCmd   `bson:"pitrestore,omitempty"`
+	Delete     *DeleteBackupCmd `bson:"delete,omitempty"`
+	DeletePITR *DeletePITRCmd   `bson:"deletePitr,omitempty"`
+	TS         int64            `bson:"ts"`
+	OPID       OPID             `bson:"-"`
 }
 
 func OPIDfromStr(s string) (OPID, error) {
@@ -158,10 +159,11 @@ func (c Cmd) String() string {
 }
 
 type BackupCmd struct {
-	Type             BackupType      `bson:"type"`
-	Name             string          `bson:"name"`
-	Compression      CompressionType `bson:"compression"`
-	CompressionLevel *int            `bson:"level,omitempty"`
+	Type             BackupType               `bson:"type"`
+	Name             string                   `bson:"name"`
+	Namespaces       []string                 `bson:"nss,omitempty"`
+	Compression      compress.CompressionType `bson:"compression"`
+	CompressionLevel *int                     `bson:"level,omitempty"`
 }
 
 func (b BackupCmd) String() string {
@@ -177,6 +179,7 @@ func (b BackupCmd) String() string {
 type RestoreCmd struct {
 	Name       string            `bson:"name"`
 	BackupName string            `bson:"backupName"`
+	Namespaces []string          `bson:"nss,omitempty"`
 	RSMap      map[string]string `bson:"rsMap,omitempty"`
 }
 
@@ -196,11 +199,12 @@ func (c ReplayCmd) String() string {
 }
 
 type PITRestoreCmd struct {
-	Name  string            `bson:"name"`
-	TS    int64             `bson:"ts"`
-	I     int64             `bson:"i"`
-	Bcp   string            `bson:"bcp"`
-	RSMap map[string]string `bson:"rsMap,omitempty"`
+	Name       string            `bson:"name"`
+	TS         int64             `bson:"ts"`
+	I          int64             `bson:"i"`
+	Bcp        string            `bson:"bcp"`
+	Namespaces []string          `bson:"nss,omitempty"`
+	RSMap      map[string]string `bson:"rsMap,omitempty"`
 }
 
 func (p PITRestoreCmd) String() string {
@@ -223,69 +227,6 @@ func (d DeleteBackupCmd) String() string {
 	return fmt.Sprintf("backup: %s, older than: %d", d.Backup, d.OlderThan)
 }
 
-type CompressionType string
-
-const (
-	CompressionTypeNone      CompressionType = "none"
-	CompressionTypeGZIP      CompressionType = "gzip"
-	CompressionTypePGZIP     CompressionType = "pgzip"
-	CompressionTypeSNAPPY    CompressionType = "snappy"
-	CompressionTypeLZ4       CompressionType = "lz4"
-	CompressionTypeS2        CompressionType = "s2"
-	CompressionTypeZstandard CompressionType = "zstd"
-)
-
-func isValidCompressionType(s string) bool {
-	switch CompressionType(s) {
-	case
-		CompressionTypeNone,
-		CompressionTypeGZIP,
-		CompressionTypePGZIP,
-		CompressionTypeSNAPPY,
-		CompressionTypeLZ4,
-		CompressionTypeS2,
-		CompressionTypeZstandard:
-		return true
-	}
-
-	return false
-}
-
-func (c CompressionType) Suffix() string {
-	switch c {
-	case CompressionTypeGZIP, CompressionTypePGZIP:
-		return ".gz"
-	case CompressionTypeLZ4:
-		return ".lz4"
-	case CompressionTypeSNAPPY:
-		return ".snappy"
-	case CompressionTypeS2:
-		return ".s2"
-	case CompressionTypeZstandard:
-		return ".zst"
-	default:
-		return ""
-	}
-}
-
-// FileCompression return compression alg based on given file extension
-func FileCompression(ext string) CompressionType {
-	switch ext {
-	default:
-		return CompressionTypeNone
-	case "gz":
-		return CompressionTypePGZIP
-	case "lz4":
-		return CompressionTypeLZ4
-	case "snappy":
-		return CompressionTypeSNAPPY
-	case "s2":
-		return CompressionTypeS2
-	case "zst":
-		return CompressionTypeZstandard
-	}
-}
-
 const (
 	PITRcheckRange       = time.Second * 15
 	AgentsStatCheckRange = time.Second * 5
@@ -491,7 +432,7 @@ func (p *PBM) setupNewDB() error {
 		},
 	)
 
-	return nil
+	return err
 }
 
 func connect(ctx context.Context, uri, appName string) (*mongo.Client, error) {
@@ -527,24 +468,26 @@ const (
 
 // BackupMeta is a backup's metadata
 type BackupMeta struct {
-	Type             BackupType           `bson:"type" json:"type"`
-	OPID             string               `bson:"opid" json:"opid"`
-	Name             string               `bson:"name" json:"name"`
-	Replsets         []BackupReplset      `bson:"replsets" json:"replsets"`
-	Compression      CompressionType      `bson:"compression" json:"compression"`
-	Store            StorageConf          `bson:"store" json:"store"`
-	MongoVersion     string               `bson:"mongodb_version" json:"mongodb_version,omitempty"`
-	StartTS          int64                `bson:"start_ts" json:"start_ts"`
-	LastTransitionTS int64                `bson:"last_transition_ts" json:"last_transition_ts"`
-	FirstWriteTS     primitive.Timestamp  `bson:"first_write_ts" json:"first_write_ts"`
-	LastWriteTS      primitive.Timestamp  `bson:"last_write_ts" json:"last_write_ts"`
-	Hb               primitive.Timestamp  `bson:"hb" json:"hb"`
-	Status           Status               `bson:"status" json:"status"`
-	Conditions       []Condition          `bson:"conditions" json:"conditions"`
-	Nomination       []BackupRsNomination `bson:"n" json:"n"`
-	Err              string               `bson:"error,omitempty" json:"error,omitempty"`
-	PBMVersion       string               `bson:"pbm_version,omitempty" json:"pbm_version,omitempty"`
-	BalancerStatus   BalancerMode         `bson:"balancer" json:"balancer"`
+	Type             BackupType               `bson:"type" json:"type"`
+	OPID             string                   `bson:"opid" json:"opid"`
+	Name             string                   `bson:"name" json:"name"`
+	Namespaces       []string                 `bson:"nss,omitempty" json:"nss,omitempty"`
+	Replsets         []BackupReplset          `bson:"replsets" json:"replsets"`
+	Compression      compress.CompressionType `bson:"compression" json:"compression"`
+	Store            StorageConf              `bson:"store" json:"store"`
+	Size             int64                    `bson:"size" json:"size"`
+	MongoVersion     string                   `bson:"mongodb_version" json:"mongodb_version,omitempty"`
+	StartTS          int64                    `bson:"start_ts" json:"start_ts"`
+	LastTransitionTS int64                    `bson:"last_transition_ts" json:"last_transition_ts"`
+	FirstWriteTS     primitive.Timestamp      `bson:"first_write_ts" json:"first_write_ts"`
+	LastWriteTS      primitive.Timestamp      `bson:"last_write_ts" json:"last_write_ts"`
+	Hb               primitive.Timestamp      `bson:"hb" json:"hb"`
+	Status           Status                   `bson:"status" json:"status"`
+	Conditions       []Condition              `bson:"conditions" json:"conditions"`
+	Nomination       []BackupRsNomination     `bson:"n" json:"n"`
+	Err              string                   `bson:"error,omitempty" json:"error,omitempty"`
+	PBMVersion       string                   `bson:"pbm_version,omitempty" json:"pbm_version,omitempty"`
+	BalancerStatus   BalancerMode             `bson:"balancer" json:"balancer"`
 	runtimeError     error
 }
 
@@ -585,11 +528,13 @@ type BackupReplset struct {
 	OplogName        string              `bson:"oplog_name,omitempty" json:"oplog_name,omitempty"`
 	StartTS          int64               `bson:"start_ts" json:"start_ts"`
 	Status           Status              `bson:"status" json:"status"`
+	IsConfigSvr      *bool               `bson:"iscs,omitempty" json:"iscs,omitempty"`
 	LastTransitionTS int64               `bson:"last_transition_ts" json:"last_transition_ts"`
 	FirstWriteTS     primitive.Timestamp `bson:"first_write_ts" json:"first_write_ts"`
 	LastWriteTS      primitive.Timestamp `bson:"last_write_ts" json:"last_write_ts"`
 	Error            string              `bson:"error,omitempty" json:"error,omitempty"`
 	Conditions       []Condition         `bson:"conditions" json:"conditions"`
+	MongodOpts       *MongodOpts         `bson:"mongod_opts,omitempty" json:"mongod_opts,omitempty"`
 }
 
 type File struct {
@@ -606,12 +551,13 @@ const (
 	StatusInit  Status = "init"
 	StatusReady Status = "ready"
 
-	StatusStarting  Status = "starting"
-	StatusRunning   Status = "running"
-	StatusDumpDone  Status = "dumpDone"
-	StatusDone      Status = "done"
-	StatusCancelled Status = "canceled"
-	StatusError     Status = "error"
+	StatusStarting   Status = "starting"
+	StatusRunning    Status = "running"
+	StatusDumpDone   Status = "dumpDone"
+	StatusPartlyDone Status = "partlyDone"
+	StatusDone       Status = "done"
+	StatusCancelled  Status = "canceled"
+	StatusError      Status = "error"
 )
 
 func (p *PBM) SetBackupMeta(m *BackupMeta) error {
@@ -733,6 +679,14 @@ func (p *PBM) ChangeRSState(bcpName string, rsName string, s Status, msg string)
 	return err
 }
 
+func (p *PBM) IncBackupSize(ctx context.Context, bcpName string, size int64) error {
+	_, err := p.Conn.Database(DB).Collection(BcpCollection).UpdateOne(ctx,
+		bson.D{{"name", bcpName}},
+		bson.D{{"$inc", bson.M{"size": size}}})
+
+	return err
+}
+
 func (p *PBM) RSSetPhyFiles(bcpName string, rsName string, f []File) error {
 	_, err := p.Conn.Database(DB).Collection(BcpCollection).UpdateOne(
 		p.ctx,
@@ -792,6 +746,7 @@ func (p *PBM) GetFirstBackup(after *primitive.Timestamp) (*BackupMeta, error) {
 
 func (p *PBM) getRecentBackup(after, before *primitive.Timestamp, sort int) (*BackupMeta, error) {
 	q := bson.D{
+		{"nss", nil},
 		{"status", StatusDone},
 		{"type", bson.M{"$ne": string(PhysicalBackup)}},
 	}
@@ -962,8 +917,18 @@ func (p *PBM) GetNodeInfo() (*NodeInfo, error) {
 	inf := &NodeInfo{}
 	err := p.Conn.Database(DB).RunCommand(p.ctx, bson.D{{"isMaster", 1}}).Decode(inf)
 	if err != nil {
-		return nil, errors.Wrap(err, "run mongo command")
+		return nil, errors.Wrap(err, "get NodeInfo")
 	}
+
+	opts := struct {
+		Parsed MongodOpts `bson:"parsed" json:"parsed"`
+	}{}
+	err = p.Conn.Database("admin").RunCommand(p.ctx, bson.D{{"getCmdLineOpts", 1}}).Decode(&opts)
+	if err != nil {
+		return nil, errors.Wrap(err, "get mongod options")
+	}
+	inf.opts = opts.Parsed
+
 	return inf, nil
 }
 
