@@ -2,6 +2,7 @@ package pbm
 
 import (
 	"fmt"
+	"sort"
 	"time"
 
 	"github.com/pkg/errors"
@@ -17,13 +18,14 @@ type RestoreMeta struct {
 	Name             string              `bson:"name" json:"name"`
 	OPID             string              `bson:"opid" json:"opid"`
 	Backup           string              `bson:"backup" json:"backup"`
+	Namespaces       []string            `bson:"nss,omitempty" json:"nss,omitempty"`
 	StartPITR        int64               `bson:"start_pitr" json:"start_pitr"`
 	PITR             int64               `bson:"pitr" json:"pitr"`
 	Replsets         []RestoreReplset    `bson:"replsets" json:"replsets"`
 	Hb               primitive.Timestamp `bson:"hb" json:"hb"`
 	StartTS          int64               `bson:"start_ts" json:"start_ts"`
 	LastTransitionTS int64               `bson:"last_transition_ts" json:"last_transition_ts"`
-	Conditions       []Condition         `bson:"conditions" json:"conditions"`
+	Conditions       Conditions          `bson:"conditions" json:"conditions"`
 	Type             BackupType          `bson:"type" json:"type"`
 	Leader           string              `bson:"l,omitempty" json:"l,omitempty"`
 }
@@ -38,16 +40,31 @@ type RestoreReplset struct {
 	LastWriteTS      primitive.Timestamp `bson:"last_write_ts" json:"last_write_ts"`
 	Nodes            []RestoreNode       `bson:"nodes,omitempty" json:"nodes,omitempty"`
 	Error            string              `bson:"error,omitempty" json:"error,omitempty"`
-	Conditions       []Condition         `bson:"conditions" json:"conditions"`
+	Conditions       Conditions          `bson:"conditions" json:"conditions"`
 	Hb               primitive.Timestamp `bson:"hb" json:"hb"`
 }
 
+type Conditions []*Condition
+
+func (b Conditions) Len() int           { return len(b) }
+func (b Conditions) Less(i, j int) bool { return b[i].Timestamp < b[j].Timestamp }
+func (b Conditions) Swap(i, j int)      { b[i], b[j] = b[j], b[i] }
+
+// Insert keeps conditions asc sorted by Timestamp
+func (b *Conditions) Insert(c *Condition) {
+	i := sort.Search(len(*b), func(i int) bool { return []*Condition(*b)[i].Timestamp >= c.Timestamp })
+	*b = append(*b, &Condition{})
+	copy([]*Condition(*b)[i+1:], []*Condition(*b)[i:])
+	[]*Condition(*b)[i] = c
+}
+
 type RestoreNode struct {
-	Name             string      `bson:"name" json:"name"`
-	Status           Status      `bson:"status" json:"status"`
-	LastTransitionTS int64       `bson:"last_transition_ts" json:"last_transition_ts"`
-	Error            string      `bson:"error,omitempty" json:"error,omitempty"`
-	Conditions       []Condition `bson:"conditions" json:"conditions"`
+	Name             string              `bson:"name" json:"name"`
+	Status           Status              `bson:"status" json:"status"`
+	LastTransitionTS int64               `bson:"last_transition_ts" json:"last_transition_ts"`
+	Error            string              `bson:"error,omitempty" json:"error,omitempty"`
+	Conditions       Conditions          `bson:"conditions" json:"conditions"`
+	Hb               primitive.Timestamp `bson:"hb" json:"hb"`
 }
 
 type TxnState string
@@ -91,7 +108,7 @@ func (p *PBM) SetCurrentOp(name string, rsName string, ts primitive.Timestamp) e
 
 func (p *PBM) SetRestoreMeta(m *RestoreMeta) error {
 	m.LastTransitionTS = m.StartTS
-	m.Conditions = append(m.Conditions, Condition{
+	m.Conditions = append(m.Conditions, &Condition{
 		Timestamp: m.StartTS,
 		Status:    m.Status,
 	})
@@ -144,7 +161,7 @@ func (p *PBM) GetLastRestore() (*RestoreMeta, error) {
 
 func (p *PBM) AddRestoreRSMeta(name string, rs RestoreReplset) error {
 	rs.LastTransitionTS = rs.StartTS
-	rs.Conditions = append(rs.Conditions, Condition{
+	rs.Conditions = append(rs.Conditions, &Condition{
 		Timestamp: rs.StartTS,
 		Status:    rs.Status,
 	})
